# This file was auto-generated by salt's setup on Wednesday, 24 August 2016 @ 16:08:31 UTC.

from salt.version import SaltStackVersion

__saltstack_version__ = SaltStackVersion(2015, 8, 12, 0, 0, 0, None)
